package net.wanji.opt.task;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import freemarker.template.TemplateException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.Constants;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.GreenwaveCrossPO;
import net.wanji.databus.dao.entity.GreenwaveInfoPO;
import net.wanji.databus.dao.mapper.CrossDataRealtimeMapper;
import net.wanji.databus.dao.mapper.GreenwaveCrossMapper;
import net.wanji.databus.dao.mapper.GreenwaveInfoMapper;
import net.wanji.opt.dao.mapper.StrategyDailyPlanInfoMapper;
import net.wanji.opt.dao.mapper.StrategyGreenOptHistMapper;
import net.wanji.opt.dao.mapper.StrategyPlanInfoMapper;
import net.wanji.opt.dto.induce.MessageParam;
import net.wanji.opt.entity.GreenwaveInduces;
import net.wanji.opt.entity.GreenwaveInducesHist;
import net.wanji.opt.entity.InduceTemplate;
import net.wanji.opt.po.StrategyGreenOptHistEntity;
import net.wanji.opt.service.induce.GreenwaveInducesHistService;
import net.wanji.opt.service.induce.GreenwaveInducesService;
import net.wanji.opt.service.induce.InduceSendService;
import net.wanji.opt.service.induce.InduceTemplateService;
import net.wanji.opt.synthesis.pojo.StrategyControlDetailList;
import net.wanji.opt.synthesis.pojo.StrategyDailyPlanInfoEntity;
import net.wanji.opt.synthesis.pojo.StrategyPlanInfoEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.text.ParseException;
import java.time.LocalDate;
import java.time.LocalTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicLong;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * @author guoliang.dong
 * @date 2024/11/21 19:18
 * @desc 诱导信息下发监测
 */
@RequiredArgsConstructor
@Component
@Slf4j
public class InducesMonitorTask {

    @Autowired
    private final GreenwaveInfoMapper greenwaveInfoMapper;

    @Autowired
    private final GreenwaveCrossMapper greenwaveCrossMapper;

    @Autowired
    private final StrategyPlanInfoMapper strategyPlanInfoMapper;

    @Autowired
    private final StrategyDailyPlanInfoMapper strategyDailyPlanInfoMapper;

    @Autowired
    private StrategyGreenOptHistMapper strategyGreenOptHistMapper;

    @Autowired
    private final CrossDataRealtimeMapper crossDataRealtimeMapper;

    @Autowired
    private final GreenwaveInducesHistService greenwaveInducesHistService;

    @Autowired
    private final GreenwaveInducesService greenwaveInducesService;

    @Autowired
    private InduceSendService induceSendService;

    @Autowired
    private InduceTemplateService induceTemplateService;

    /**
     * #绿波调度计划扫描周期
     * 2 分钟 120000
     */
    @Scheduled(fixedRate = 120000)
    public void refresh() throws Exception {
        long start = System.currentTimeMillis();
        AtomicLong synCount = new AtomicLong();
        try {
            log.info("每5分钟刷新绿波优化下发数据...................");
            List<GreenwaveInfoPO> greenwaveInfoPOList=greenwaveInfoMapper.selectAll();
            List<GreenwaveInducesHist> greenwaveInducesHistss = greenwaveInducesHistService.list();

            for(GreenwaveInfoPO greenwaveInfoPO :greenwaveInfoPOList) {
                AtomicBoolean flag = new AtomicBoolean(false);//是否已发布优化
                AtomicReference<Integer> strategyId = new AtomicReference<Integer>(0);
                //1、查询所有绿波带路口信息，获取路口下的绿波策略计划
                List<GreenwaveCrossPO> greenwaveCrossList = greenwaveCrossMapper.selectByGreenwaveId(greenwaveInfoPO.getId());
                greenwaveCrossList.stream().forEach(o -> {
                    //System.out.println("绿波路口--->"+greenwaveInfoPO.getId()+"=="+o.getCrossId());
                    //创建路口策略列表
                    List<StrategyPlanInfoEntity> strategyPlanInfoEntityList = strategyPlanInfoMapper.selectByCrossId(o.getCrossId());
                    //strategyPlanInfoEntityList.stream().filter(x->Objects.nonNull(x.getCrossId()) && x.getCrossId().equals(o.getCrossId()) ).forEach(x->
                    for (StrategyPlanInfoEntity strategyPlanInfoEntity : strategyPlanInfoEntityList) {
                        System.out.println("策略计划--->" + greenwaveInfoPO.getId() + "==" + strategyPlanInfoEntity.getPlanDetails());
                        //2、检查绿波时段，判断是否在绿波时段内，如果不在，则判断是否执行过上屏操作，如果执行过，则调用下屏服务（此出屏幕自动处理，无需逻辑）
                        //[{"dailyPlanId":1,"weeks":[1,2,3,4,5]}]
                        List<StrategyControlDetailList.ExecutePlan.PlanDetail> planDetails = new ArrayList<>();
                        try {
                            planDetails = JacksonUtils.getInstance().readValue(strategyPlanInfoEntity.getPlanDetails(), new TypeReference<List<StrategyControlDetailList.ExecutePlan.PlanDetail>>() {
                            });
                        } catch (JsonProcessingException e) {
                            log.error("策略周计划详情转换失败：" + strategyPlanInfoEntity.getPlanId() + "---" + e.getMessage());
                        }
                        //处理日计划的时间拼合处理
                        if (!CollectionUtils.isEmpty(planDetails)) {
                            for (StrategyControlDetailList.ExecutePlan.PlanDetail planDetail : planDetails) {
                                Integer dailyPlanId = planDetail.getDailyPlanId();
                                List<Integer> weeks = planDetail.getWeeks();
                                //获取当前周几，判断是否在时间范围内
                                if (weeks.contains(LocalDate.now().getDayOfWeek().getValue())) {
                                    LambdaQueryWrapper<StrategyDailyPlanInfoEntity> dailyPlanQueryWrapper = new LambdaQueryWrapper<>();
                                    dailyPlanQueryWrapper.eq(StrategyDailyPlanInfoEntity::getCrossId, strategyPlanInfoEntity.getCrossId());
                                    dailyPlanQueryWrapper.eq(StrategyDailyPlanInfoEntity::getDailyPlanId, dailyPlanId);
                                    List<StrategyDailyPlanInfoEntity> dailyPlanInfoEntities = strategyDailyPlanInfoMapper.selectList(dailyPlanQueryWrapper);
                                    dailyPlanInfoEntities.stream().forEach(p -> {
                                        System.out.println("plan detail->" + p.getDailyPlanDetails());
                                        List<StrategyControlDetailList.DailyPlan.DailyPlanDetail> dailyPlanDetails = new ArrayList<>();
                                        try {
                                            dailyPlanDetails = JacksonUtils.getInstance().readValue(p.getDailyPlanDetails(), new TypeReference<List<StrategyControlDetailList.DailyPlan.DailyPlanDetail>>() {
                                            });
                                        } catch (JsonProcessingException e) {
                                            log.error("策略日计划详情转换失败：" + p.getDailyPlanId() + "---" + e.getMessage());
                                        }
                                        for (StrategyControlDetailList.DailyPlan.DailyPlanDetail dailyPlanDetail : dailyPlanDetails) {
                                            //[{"strategy":1,"times":["00:00","20:00"]},{"strategy":2,"times":["20:00","23:59"]}]
                                            if (!CollectionUtils.isEmpty(dailyPlanDetail.getTimes())) {
                                                LocalTime startTime = LocalTime.parse(dailyPlanDetail.getTimes().get(0));
                                                LocalTime endTime = LocalTime.parse(dailyPlanDetail.getTimes().get(1));
                                                // 判断当前时间是否在范围内
                                                if (LocalTime.now().isAfter(startTime) && LocalTime.now().isBefore(endTime)) {
                                                    System.out.println("当前时间在范围内: " + LocalTime.now());
                                                    //System.out.println("绿波路口--->"+greenwaveInfoPO.getId()+"=="+o.getCrossId());
                                                    flag.set(true);
                                                    strategyId.set(p.getDailyPlanId());
                                                }
                                            }
                                        }
                                    });
                                }
                            }
                        }
                    }
                });
                //3、如果在绿波时段内，未上屏，则获取固定配时绿波信息
                //4、获取优化绿波方案，如果不存在，则执行固定绿波方案
                //如果在绿波时段内，未上屏，则获取固定配时绿波信息
                if (flag.get()) {
                    List<StrategyGreenOptHistEntity> strategyGreenOptHistEntitiesList = strategyGreenOptHistMapper.selectByGreenId(String.valueOf(greenwaveInfoPO.getId())).stream().distinct().collect(Collectors.toList());
                    if (Objects.nonNull(strategyGreenOptHistEntitiesList)) {
                        for (StrategyGreenOptHistEntity greenOptHistEntity : strategyGreenOptHistEntitiesList) {
                            Integer controlMethod = greenOptHistEntity.getControlMethod();
                            if (Objects.equals(-1, controlMethod)) {
                                continue;
                            }
//                            LambdaQueryWrapper<GreenwaveInducesHist> greenwaveInducesHistQueryWrapper = new LambdaQueryWrapper<>();
//                            greenwaveInducesHistQueryWrapper.eq(GreenwaveInducesHist::getGreenId, greenwaveInfoPO.getId());
//                            greenwaveInducesHistQueryWrapper.eq(GreenwaveInducesHist::getDir, getDir(greenOptHistEntity.getDir()));
                            List<GreenwaveInducesHist> greenwaveInducesHistList = greenwaveInducesHistss.stream().filter(g -> g.getGreenId().equals(greenwaveInfoPO.getId()) && g.getDir().equals(getDir(greenOptHistEntity.getDir()))).collect(Collectors.toList());
                            GreenwaveInducesHist greenwaveInducesHist = null;
                            if (Objects.isNull(greenwaveInducesHistList) || greenwaveInducesHistList.size() < 1) {
                                greenwaveInducesHist = new GreenwaveInducesHist();
                                greenwaveInducesHist.setCreateTime(greenOptHistEntity.getCreateTime());
                                greenwaveInducesHist.setGreenId(greenwaveInfoPO.getId());
                                greenwaveInducesHist.setDir(getDir(greenOptHistEntity.getDir()));
                                greenwaveInducesHistService.save(greenwaveInducesHist);
                            } else {
                                greenwaveInducesHist = greenwaveInducesHistList.get(0);
                            }
                            greenwaveInducesHist.setStrategyId(strategyId.get());
                            greenwaveInducesHist.setStrategyName(greenwaveInfoPO.getName());//当前策略名称
                            if (StringUtils.isNotBlank(greenOptHistEntity.getControlTime()) && greenOptHistEntity.getControlDuration() > 0) {
                                greenwaveInducesHist.setControlOptTimes(greenOptHistEntity.getControlTime() + "|" + DateUtil.format(DateUtil.addSecond(greenOptHistEntity.getControlTime(), greenOptHistEntity.getControlDuration()), Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND));//执行时段
                            }
                            greenwaveInducesHist.setType(greenOptHistEntity.getType());// 执行策略
                            String upDown = "";
                            // 计算绿波速度变化趋势
                            if (Objects.nonNull(greenwaveInducesHist.getMinSpeed()) && Objects.nonNull(greenwaveInducesHist.getMaxSpeed()) && Objects.nonNull(greenOptHistEntity.getMinSpeed()) && Objects.nonNull(greenOptHistEntity.getMaxSpeed())) {
                                if (greenwaveInducesHist.getMinSpeed() > greenOptHistEntity.getMinSpeed() || greenwaveInducesHist.getMaxSpeed() > greenOptHistEntity.getMaxSpeed()) {
                                    upDown = "";
                                }
                                if (greenwaveInducesHist.getMinSpeed() < greenOptHistEntity.getMinSpeed() || greenwaveInducesHist.getMaxSpeed() < greenOptHistEntity.getMaxSpeed()) {
                                    upDown = "";
                                }
                            }
                            greenwaveInducesHist.setMinSpeed(greenOptHistEntity.getMinSpeed());
                            greenwaveInducesHist.setMaxSpeed(greenOptHistEntity.getMaxSpeed());
//                            greenwaveInducesHist.setDir(greenOptHistEntity.getDirType());
                            //5、获取绿波信息，调用上屏服务；发布互联网诱导方案
                            //[{"crossId":"13MOD0B5SI0","phaseStartTime":32.0,"phaseEndTime":122.0,"greenStartTime":35.0,"speed":"39.71","offset":208.0,"travelTime":57.0,"distance":"628.81"},{"crossId":"13MQJ0B5SI0","phaseStartTime":43.0,"phaseEndTime":158.0,"greenStartTime":0.0,"speed":"35.54","offset":69.0,"travelTime":42.0,"distance":"414.62"},{"crossId":"13MS20B5SI0","phaseStartTime":32.0,"phaseEndTime":142.0,"greenStartTime":8.0,"speed":"22.72","offset":114.0,"travelTime":116.0,"distance":"732.00"},{"crossId":"13MUK0B5SH0","phaseStartTime":0.0,"phaseEndTime":135.0,"greenStartTime":0.0,"speed":"42.33","offset":50.0,"travelTime":45.0,"distance":"529.16"},{"crossId":"13N0F0B5SH0","phaseStartTime":25.0,"phaseEndTime":156.0,"greenStartTime":70.0,"speed":"40.67","offset":0.0,"travelTime":40.0,"distance":"451.89"},{"crossId":"13N200B5SH0","phaseStartTime":23.0,"phaseEndTime":150.0,"greenStartTime":0.0,"speed":"-1.00","offset":112.0,"travelTime":-1.0,"distance":"-1.00"}]
//                            List<GreenBeltInfoVO.CrossGreenDetail> crossGreenDetailList = new ArrayList<>();
//                            try {
//                                crossGreenDetailList = JacksonUtils.getInstance().readValue(greenOptHistEntity.getCrossGreenDetail(), new TypeReference<List<GreenBeltInfoVO.CrossGreenDetail>>() {});
//                            } catch (JsonProcessingException e) {
//                                log.error("绿波详情转换失败：" + greenOptHistEntity.getGreenId() + "---" + e.getMessage());
//                            }
                            //greenwaveInducesHistService.saveOrUpdate(greenwaveInducesHist);
                            ////自动发送诱导消息
                            MessageParam messageParam = new MessageParam();
                            messageParam.setFlg(1);
                            messageParam.setGreenId(greenwaveInfoPO.getId());

                            messageParam.setContents(new String[]{(upDown + (Objects.nonNull(greenOptHistEntity.getMinSpeed()) ? greenOptHistEntity.getMinSpeed().intValue() + "-" : "") + (Objects.nonNull(greenOptHistEntity.getMaxSpeed()) ? greenOptHistEntity.getMaxSpeed().intValue() : "")) + " km/h"});
                            messageParam.setType("TFMH");
                            try {
                                if (greenwaveInducesHist.getStatus() == 0 || greenwaveInducesHist.getModifyTime().before(greenOptHistEntity.getCreateTime())) {
                                    LambdaQueryWrapper<GreenwaveInduces> greenwaveInducesQueryWrapper = new LambdaQueryWrapper<>();
                                    greenwaveInducesQueryWrapper.eq(GreenwaveInduces::getGreenId, greenwaveInfoPO.getId());
                                    greenwaveInducesQueryWrapper.eq(GreenwaveInduces::getDir, getDir(greenOptHistEntity.getDir()));
                                    List<GreenwaveInduces> greenwaveInducesList = greenwaveInducesService.list(greenwaveInducesQueryWrapper);
                                    for (GreenwaveInduces greenwaveCross : greenwaveInducesList) {
                                        LambdaQueryWrapper<InduceTemplate> induceTemplateQueryWrapper = new LambdaQueryWrapper<>();
                                        induceTemplateQueryWrapper.eq(InduceTemplate::getEquipCode, greenwaveCross.getEquipCode());
                                        induceTemplateQueryWrapper.eq(InduceTemplate::getDefaultTemplate, 1);//筛选默认模板进行自动发送，目前屏幕轮播参数无效
                                        List<InduceTemplate> induceTemplateList = induceTemplateService.list(induceTemplateQueryWrapper);
                                        for (InduceTemplate induceTemplate : induceTemplateList) {
                                            messageParam.setTemplateId(induceTemplate.getId());
                                            messageParam.setInduceId(greenwaveCross.getId());
                                            messageParam.setEquipCode(greenwaveCross.getEquipCode());
                                            messageParam.setPlayorder(1);//induceTemplateList.size()
                                            messageParam.setDuration(greenwaveCross.getDuration());
                                            if (greenwaveInducesHist.getControlOptTimes().split("\\|").length > 0 && DateUtil.isBetween(new Date(), DateUtil.parse(greenwaveInducesHist.getControlOptTimes().split("\\|")[0], Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND), DateUtil.parse(greenwaveInducesHist.getControlOptTimes().split("\\|")[1], Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND))) {
                                                messageParam.setStartTime(greenwaveInducesHist.getControlOptTimes().split("\\|")[0]);
                                                messageParam.setEndTime(greenwaveInducesHist.getControlOptTimes().split("\\|")[1]);
                                            } else {
                                                messageParam.setStartTime(DateUtil.format(new Date(), Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND));
                                                messageParam.setEndTime(DateUtil.format(System.currentTimeMillis() + 1000 * 60, Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND));
                                            }
                                            induceSendService.send(messageParam);
                                            Thread.sleep(10000);
                                        }
                                    }
                                    greenwaveInducesHist.setStatus(1);
                                    greenwaveInducesHist.setModifyTime(greenOptHistEntity.getCreateTime());
                                }
                            } catch (IOException e) {
                                e.printStackTrace();
                            } catch (TemplateException e) {
                                log.info("发送上屏信息异常" + e.getMessage());
                            }
                            greenwaveInducesHistService.updateById(greenwaveInducesHist);
                            synCount.getAndIncrement();
                            log.info("同步绿波状态信息成功->" + greenwaveInfoPO.getName() + greenwaveInducesHist.getDir());
                        }
                    }
                }
                //如果没有绿波方案则调用下屏服务
                else {
                    LambdaQueryWrapper<GreenwaveInducesHist> greenwaveInducesHistQueryWrapper = new LambdaQueryWrapper<>();
                    greenwaveInducesHistQueryWrapper.eq(GreenwaveInducesHist::getGreenId, greenwaveInfoPO.getId());
                    List<GreenwaveInducesHist> greenwaveInducesHistList = greenwaveInducesHistService.list(greenwaveInducesHistQueryWrapper);
                    greenwaveInducesHistList.stream().forEach(n -> {
                        //调用下屏
//                        try {
                            if (n.getStatus() == 1) {
                                MessageParam messageParam = new MessageParam();
                                messageParam.setFlg(2);
                                LambdaQueryWrapper<GreenwaveInduces> greenwaveInducesQueryWrapper = new LambdaQueryWrapper<>();
                                greenwaveInducesQueryWrapper.eq(GreenwaveInduces::getGreenId, greenwaveInfoPO.getId());
                                List<GreenwaveInduces> greenwaveInducesList = greenwaveInducesService.list(greenwaveInducesQueryWrapper);
                                for (GreenwaveInduces greenwaveInduces : greenwaveInducesList) {
                                    messageParam.setEquipCode(greenwaveInduces.getEquipCode());
                                    messageParam.setSourceId(greenwaveInduces.getSourceId());
                                    messageParam.setGreenId(greenwaveInduces.getGreenId());
                                    messageParam.setInduceId(greenwaveInduces.getId());
                                    messageParam.setType("TFMH");
                                    //屏蔽此处功能，因为上屏的时候可以设置生效时间
                                    try {
                                        induceSendService.send(messageParam);
                                        //保存绿波状态
                                        n.setStatus(0);
                                        n.setModifyTime(new Date());
                                        Thread.sleep(1000);
                                    } catch (TemplateException e) {
                                        log.info("发送下屏信息异常" + e.getMessage());
                                    } catch (IOException e) {
                                        log.info(e.getMessage());
                                    } catch (InterruptedException e) {
                                        e.printStackTrace();
                                    }
                                    log.info("发送下屏信息成功.");
                                }
                            }
//                        } catch (Exception e) {
//                            log.error("发送下屏信息失败"+e.getMessage());
//                        }
                        greenwaveInducesHistService.updateById(n);
                        log.info("同步绿波状态信息(下屏)->" + greenwaveInfoPO.getName() + "####" + n.getDir());
//                        synCount.getAndIncrement();
                    });
                }
            }

            // 下屏逻辑，如果绿波历史表结束优化control_method=-1， 在15分钟内，下屏
            try {
                List<MessageParam> entities = strategyGreenOptHistMapper.selectGreenOptEndTimeEquipCode();
                if (!CollectionUtils.isEmpty(entities)) {
                    for (MessageParam messageParam : entities) {
                        String endTime = messageParam.getEndTime();
                        Integer greenId = messageParam.getGreenId();
                        String equipCode = messageParam.getEquipCode();
                        Date parse = DateUtil.parse(endTime, Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND);
                        long current = System.currentTimeMillis();
                        // 结束时间小于服务器时间15分钟不在下屏
                        long time = parse.getTime();
                        if (time - current < 15 * 60 * 1000 || StringUtils.isBlank(equipCode)) {
                            continue;
                        }
                        messageParam.setFlg(2);
                        messageParam.setType("TFMH");
                        induceSendService.send(messageParam);
                        log.info("当前绿波：{}， 诱导屏编号：{}， 下屏成功", greenId, equipCode);
                        Thread.sleep(500);
                    }
                }
            } catch (Exception e) {
                log.error("通过绿波优化结束时间下屏异常：{}", e);
            }

        } catch (Exception e) {
            throw new Exception("每5分钟刷新绿波下发状态失败", e);
        } finally {
            log.info("当前运行同步绿波下发状态数据，耗时：{}ms,共同步{}条数据。", System.currentTimeMillis() - start, synCount);
        }
    }

    /**
     * 根据角度获取方向
     * @param dirType
     * 路段走向:1 南向北
     * 2 西向东
     * 3 北向南
     * 4 东向西
     * @return
     */
    private Integer getDir(String dirType) {
        if (dirType.equals("e2w")) { //东
            return 4;
        } else if (dirType.equals("s2n")) {//南
            return 1;
        } else if (dirType.equals("w2e")) {//西
            return 2;
        } else if (dirType.equals("n2s")) {//北
            return 3;
        }
        return null;
    }


    public static void main(String []args) throws ParseException {
    // 获取当前时间
//        LocalTime now = LocalTime.now();
//        List<String> timesList=new ArrayList();
//        timesList.add("00:00");
//        timesList.add("23:59");
//        // 定义时间范围
//        LocalTime startTime = LocalTime.parse(timesList.get(0));
//        LocalTime endTime = LocalTime.parse(timesList.get(1));
//
//        // 判断当前时间是否在范围内
//        if (now.isAfter(startTime) && now.isBefore(endTime)) {
//            System.out.println("当前时间在范围内: " + now);
//        } else if (now.equals(startTime) || now.equals(endTime)) {
//            System.out.println("当前时间恰好在范围边界: " + now);
//        } else {
//            System.out.println("当前时间不在范围内: " + now);
//        }

//        System.out.print(new Date().before(DateUtil.parse("2024-12-04", Constants.DATE_FORMAT.E_DATE_FORMAT_DAY)));

    }
}
